/*
 * Kneron Application general functions
 *
 * Copyright (C) 2021 Kneron, Inc. All rights reserved.
 *
 */

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "model_type.h"
#include "kmdw_console.h"

#include "kmdw_inference_app.h"
#include "demo_customize_inf_single_model.h"

void demo_customize_inf_single_model(uint32_t job_id, void *inf_input_buf)
{
    // 'inf_input_buf' and 'inf_result_buf' are provided by kdp2 middleware
    // the content of 'inf_input_buf' is transmitted from host SW = header + image
    // 'inf_result_buf' is used to carry inference result back to host SW = header + inferernce result (from ncpu/npu)

    // now get an available free result buffer
    // normally the begin part of result buffer should contain app-defined result header
    // and the rest is for ncpu/npu inference output data
    int result_buf_size;
    void *inf_result_buf = kmdw_inference_app_result_get_free_buffer(&result_buf_size);

    demo_customize_inf_single_model_header_t *input_header = (demo_customize_inf_single_model_header_t *)inf_input_buf;
    demo_customize_inf_single_model_result_t *output_result = (demo_customize_inf_single_model_result_t *)inf_result_buf;

    // config image preprocessing and model settings
    kmdw_inference_app_config_t inf_config;
    memset(&inf_config, 0, sizeof(kmdw_inference_app_config_t)); // for safety let default 'bool' to 'false'

    // image buffer address should be just after the header
    inf_config.image_buf = (void *)((uint32_t)input_header + sizeof(demo_customize_inf_single_model_header_t));
    inf_config.image_width = input_header->width;
    inf_config.image_height = input_header->height;
    inf_config.image_channel = 3;                                       // assume RGB565
    inf_config.image_format = KP_IMAGE_FORMAT_RGB565;                   // assume RGB565
    inf_config.image_norm = KP_NORMALIZE_KNERON;                        // this depends on model
    inf_config.image_resize = KP_RESIZE_ENABLE;                         // enable resize
    inf_config.image_padding = KP_PADDING_CORNER;                       // enable padding on corner
    inf_config.model_id = TINY_YOLO_V3_224_224_3;                       // this depends on model
    inf_config.ncpu_result_buf = (void *)&(output_result->yolo_result); // give result buffer for ncpu/npu, callback will carry it

    // run preprocessing and inference, trigger ncpu/npu to do the work
    // if enable_parallel=true (works only for single model), result callback is needed
    // however if inference error then no callback will be invoked
    int inf_status = kmdw_inference_app_execute(&inf_config);

    // header_stamp is a must to correctly transfer result data back to host SW
    output_result->header_stamp.magic_type = KDP2_MAGIC_TYPE_INFERENCE;
    output_result->header_stamp.total_size = sizeof(demo_customize_inf_single_model_result_t);
    output_result->header_stamp.job_id = job_id;
    output_result->header_stamp.status_code = inf_status;

    // send output result buffer back to host SW
    kmdw_inference_app_result_enqueue((void *)output_result, result_buf_size, false);
}
